@@ -3,9 +3,10 @@
     Arkivist
 """
 import json
+import requests
 
 class Arkivist:
-    def __init__(self, filepath, indent=4, sort=False, reverse=False, autosave=True):
+    def __init__(self, filepath="", indent=4, sort=False, reverse=False, autosave=True):
         """
             Read and prepare the JSON/Dictionary object.
             ...
@@ -21,14 +22,52 @@ class Arkivist:
                 sorts the dictionary in reverse
             autosave: boolean
                 save dictionary to JSON file after every update
+            save_to_file: boolean
+                save dictionary to JSON file, else just keep in memory
         """
+        self.save_to_file = True
         self.filepath = filepath
+        if filepath == "": self.save_to_file = False
         self.collection = read(filepath)
         self.indent = indent
         self.sort = sort
         self.reverse = reverse
         self.autosave = autosave
     
+    def update(self, data):
+        """
+            Formats and saves the dictionary into a JSON file.
+            ...
+            Parameters
+            ---
+            data: dictionary
+                dictionary object
+        """
+        self.collection.update(data)
+        if self.autosave and self.save_to_file:
+            update_file(self.filepath, self.collection, self.indent, self.sort, self.reverse)
+        return self
+    
+    def set(self, data):
+        """ Duplicate for update function. """
+        return self.update(data)
+
+    def fetch(self, url):
+        """
+            Get JSON data from a web API through HTTP/HTTPS GET method.
+            ...
+            Parameters
+            ---
+            url: str
+                any valid URL string
+        """
+        try:
+            with requests.get(url) as source:
+                self.collection = source.json()
+        except:
+            pass
+        return self
+    
     def load(self, data):
         """
             Replaces the existing dictionary with another specified dictionary,
@@ -49,6 +88,17 @@ class Arkivist:
                 pass
         return self
     
+    def invert(self):
+        """ Inverts the dictionary """
+        try:
+            # hashable keys / values
+            self.collection = dict(zip(self.collection.values(), self.collection.keys()))
+            if self.autosave and self.save_to_file:
+                update_file(self.filepath, self.collection, self.indent, self.sort, self.reverse)
+        except:
+            pass
+        return self
+    
     def flatten(self):
         """  Flattens a nested dictionary. """
         self.collection = flattener(self.collection)
@@ -85,31 +135,6 @@ class Arkivist:
         """
         return self.collection.get(key, fallback)
     
-    def set(self, data):
-        """
-            Formats and saves the dictionary into a JSON file.
-            ...
-            Parameters
-            ---
-            data: dictionary
-                dictionary object
-        """
-        self.collection.update(data)
-        if self.autosave:
-            update(self.filepath, self.collection, self.indent, self.sort, self.reverse)
-        return self
-    
-    def invert(self):
-        """ Inverts the dictionary """
-        try:
-            # hashable keys / values
-            self.collection = dict(zip(self.collection.values(), self.collection.keys()))
-            if self.autosave:
-                update(self.filepath, self.collection, self.indent, self.sort, self.reverse)
-        except:
-            pass
-        return self
-    
     def keys(self):
         """ Get all the keys of the ditionary """
         return list(self.collection.keys())
@@ -118,6 +143,18 @@ class Arkivist:
         """ Get all the values of the ditionary """
         return list(self.collection.values())
     
+    def is_empty(self):
+        """ Check if dictionary is empty """
+        if len(self.collection) == 0:
+            return True
+        return False
+    
+    def is_not_empty(self):
+        """ Check if dictionary is not empty """
+        if len(self.collection) == 0:
+            return False
+        return True
+    
     def count(self):
         """ Count the number of entries in the dictionary """
         return len(self.collection)
@@ -125,8 +162,8 @@ class Arkivist:
     def clear(self):
         """ Clears the dictionary """
         self.collection = {}
-        if self.autosave:
-            update(self.filepath, self.collection, self.indent, self.sort, self.reverse)
+        if self.autosave and self.save_to_file:
+            update_file(self.filepath, self.collection, self.indent, self.sort, self.reverse)
         return self
     
     def replace(self, collection):
@@ -140,23 +177,26 @@ class Arkivist:
         """
         if type(collection) == dict:
             self.collection = collection
-            if self.autosave:
-                update(self.filepath, self.collection, self.indent, self.sort, self.reverse)
+            if self.autosave and self.save_to_file:
+                update_file(self.filepath, self.collection, self.indent, self.sort, self.reverse)
         return self
 
     def save(self, filepath=""):
         """
             Save and formats the dictionary into a JSON file.
-            Doesn't change the original filepath
+            Doesn't change the original filepath.
+            If no filepath and savepath is set, write to fallback filepath.
             ...
             Parameters
             ---
             filepath: string
                 any filepath, if empty defaults to previously set filepath
         """
+        self.save_to_file = True
         savepath = self.filepath
         if filepath != "": savepath = filepath
-        update(savepath, self.collection, self.indent, self.sort, self.reverse)
+        if savepath == "": savepath = "arkivist.data.json"
+        update_file(savepath, self.collection, self.indent, self.sort, self.reverse)
         return self
 
 def flattener(data):
@@ -189,7 +229,7 @@ def flattener(data):
     flatten(data)
     return out
 
-def update(filepath, data, indent=4, sort=False, reverse=False):
+def update_file(filepath, data, indent=4, sort=False, reverse=False):
     """
         Formats and saves the dictionary into a JSON file.
         ...
